import mutex

class Error(Exception):
	pass


class WordTokenizer:
	SEP = 0
	CHR = 1
	COLLECT = 0
	IGNORE = 1
	SHIPOUT = 2
	def __init__(self):
		self.tf = {
			'Start': {
				self.SEP: (self.IGNORE, 'Start'),
				self.CHR: (self.COLLECT, 'Collect'),
			},
			'Collect': {
				self.SEP: (self.SHIPOUT, 'Start'),
				self.CHR: (self.COLLECT, 'Collect'),
			},
		}
		self.state = 'Start'
		self.collector = []

	def feed(self, ch):
		if ch.isalnum():
			chClass = self.CHR
		else:
			chClass = self.SEP
		action, self.state = self.tf[self.state][chClass]
		if action==self.COLLECT:
			self.collector.append(ch)
		elif action==self.SHIPOUT:
			val = ''.join(self.collector)
			self.collector = []
			return val

	def finish(self):
		if self.collector:
			return ''.join(self.collector)


class WordIterFile:
	iterBufSize = 8192

	def __init__(self, inFile):
		self.inFile = inFile
		self.mutex = mutex.mutex()
		self.tokenizer = WordTokenizer()

	def iterwords(self):
		if not self.mutex.testandset():
			raise Error, "Can only word-iterate over WordIterFile once."
		self.inFile.seek(0)
		while 1:
			buf = self.inFile.read(self.iterBufSize)
			if not buf:
				w = self.tokenizer.finish()
				if w:
					yield w
				self.mutex.unlock()
				raise StopIteration
			for c in buf:
				w = self.tokenizer.feed(c)
				if w:
					yield w


if __name__=="__main__":
	for word in WordIterFile(open("mar1810.txt")).iterwords():
		print word
